#ifdef _WIN32
#include <windows.h>
#include <windowsx.h>
#else
#include <sys/time.h>
#include <time.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include "string.h"
#include "ctype.h"
#include "math.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_image.h"
#include "SDL_ttf.h"

#include "auxiliar.h"
#include "BList.h"
#include "BWindow.h"

#include "mog2.h"
#include "MOG2_auxiliar.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif


extern int surface_flags;


void draw_mouse_pointer(SDL_Surface *sfc)
{
	int pointer[256]={3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
					  3,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
					  3,1,3,0,0,0,0,0,0,0,0,0,0,0,0,0,
					  3,1,1,3,0,0,0,0,0,0,0,0,0,0,0,0,
					  3,1,1,1,3,0,0,0,0,0,0,0,0,0,0,0,
					  3,1,1,1,1,3,0,0,0,0,0,0,0,0,0,0,
					  3,1,1,1,3,3,3,0,0,0,0,0,0,0,0,0,
					  3,1,3,1,3,0,0,0,0,0,0,0,0,0,0,0,
					  3,3,0,3,1,3,0,0,0,0,0,0,0,0,0,0,
					  3,0,0,3,1,3,0,0,0,0,0,0,0,0,0,0,
					  0,0,0,0,3,1,3,0,0,0,0,0,0,0,0,0,
					  0,0,0,0,3,1,3,0,0,0,0,0,0,0,0,0,
					  0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,
					  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
					  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
					  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	int mx,my;
	int i,j;

	SDL_SetClipRect(sfc,0);

	SDL_GetMouseState(&mx, &my);

	SDL_LockSurface(sfc);
	for(i=0;i<16;i++) {
		for(j=0;j<16;j++) {
			switch(pointer[i*16+j]) {
			case 1:
				putpixel(sfc,mx+j,my+i,0x00ffffff);
				break;
			case 2:
				putpixel(sfc,mx+j,my+i,0x000000ff);
				break;
			case 3:
				putpixel(sfc,mx+j,my+i,0x00000000);
				break;
			} /* switch */ 
		} /* for */ 
	} /* for */ 
	SDL_UnlockSurface(sfc);
} /* draw_mouse_pointer */ 


void draw_button(SDL_Surface *sfc,int x,int y,int dx,int dy,char *text,TTF_Font *font,bool selected)
{
	Uint32 border_color=0x0000007f;
	Uint32 fill_color=0x000000cf;

	if (selected) border_color=0x00ffffff;

	{
		SDL_Rect r;

		r.x=x+1;
		r.y=y+1;
		r.w=dx-2;
		r.h=dy-2;
		SDL_FillRect(sfc,&r,fill_color);
	}
	draw_rectangle(sfc,x,y,dx-1,dy-1,border_color);

	print_centered(text,font,255,255,255,sfc,x+dx/2,y+dy/2-TTF_FontHeight(font)/2);
} /* draw_button */ 


void draw_button2(SDL_Surface *sfc,BButton *but,TTF_Font *font,bool selected)
{
	Uint32 border_color=0x0000007f;
	Uint32 fill_color=0x000000cf;

	if (selected) border_color=0x00ffffff;

	{
		SDL_Rect r;

		r.x=but->m_x+1;
		r.y=but->m_y+1;
		r.w=but->m_dx-2;
		r.h=but->m_dy-2;
		SDL_FillRect(sfc,&r,fill_color);
	}
	draw_rectangle(sfc,but->m_x,but->m_y,but->m_dx-1,but->m_dy-1,border_color);

	print_centered(but->m_text,font,255,255,255,sfc,but->m_x+but->m_dx/2,but->m_y+but->m_dy/2-TTF_FontHeight(font)/2);
} /* draw_button2 */ 


void encode_string(char *str,char **result)
{
	int c=0,i,j;

	if (str==0) {
		*result=new char[3];
		(*result)[0]='\"';
		(*result)[1]='\"';
		(*result)[2]=0;
	} else {
		for(i=0;str[i]!=0;i++) if (str[i]=='\"' || str[i]=='\\') c++;
		*result=new char[strlen(str)+3+c];

		j=0;
		(*result)[j++]='\"';
		for(i=0;str[i]!=0;i++) {
			if (str[i]=='\"') {
				(*result)[j++]='\\';
				(*result)[j++]='\"';
			} else {
				if (str[i]=='\\') {
					(*result)[j++]='\\';
					(*result)[j++]='\\';
				} else {
					(*result)[j++]=str[i];
				} /* if */ 
			} /* if */ 
		} /* for */ 
		(*result)[j++]='\"';
		(*result)[j++]=0;
	} /* if */ 
} /* encode_String */ 


void decode_string(char *str,char **result)
{
	int i,j;
	char *tmp=new char[strlen(str)+1];

	j=0;
	for(i=0;str[i]!=0;i++) {
		if (str[i]!='\"') {
			if (str[i]=='\\') {
				i++;
				tmp[j++]=str[i];
			} else {
				tmp[j++]=str[i];
			} /* if */ 			
		} /* if */ 
	} /* for */ 
	tmp[j++]=0;

	*result=new char[strlen(tmp)+1];
	strcpy(*result,tmp);
	delete []tmp;
} /* decode_string */ 


bool load_string(FILE *fp,char **result)
{
	int i,j=0;
	int len=128;
	char *tmp=new char[len],*tmp2;
	int c;
	int state=0;

	c=fgetc(fp);
	for(i=0;state<2;) {
		if (c!='\"') {
			if (state==1) {
				if (c=='\\') {
					c=fgetc(fp);
					tmp[j++]=c;
				} else {
					tmp[j++]=c;
				} /* if */ 
			} /* if */ 
			c=fgetc(fp);
		} else {
			state++;
			if (state<2) {
				if (feof(fp)) {
					delete []tmp;
					return false;
				} /* if */ 
				c=fgetc(fp);
			} /* if */ 
		} /* if */ 
		if (j>=len) {
			len+=128;
			tmp2=new char[len];
			strcpy(tmp2,tmp);
			delete []tmp;
			tmp=tmp2;
			tmp2=0;
		} /* if */ 
	} /* for */ 
	tmp[j++]=0;

	*result=new char[strlen(tmp)+1];
	strcpy(*result,tmp);
	delete []tmp;

	return true;
} /* load_string */ 



void rotate_coordinates(int x,int y,float angle,int *rx,int *ry)
{
	double c=cos(-angle);
	double s=sin(-angle);

	*rx=int(c*x-s*y);
	*ry=int(s*x+c*y);
} /* rotate_coordinates */ 



/* This function is taken directly from the SDL_rotozoom library in order to compute exactly the same	*/ 
/* surface size as the rotozoom function: */ 

void rotate_surface_size(int width, int height, double radangle, double zoom, int *dstwidth, int *dstheight)
{
    double x, y, cx, cy, sx, sy;
	double sanglezoom,canglezoom;
    int dstwidthhalf, dstheighthalf;

    /*
     * Determine destination width and height by rotating a centered source box 
     */
    sanglezoom = sin(radangle);
    canglezoom = cos(radangle);
    sanglezoom *= zoom;
    canglezoom *= zoom;
    x = width / 2;
    y = height / 2;
    cx = canglezoom * x;
    cy = canglezoom * y;
    sx = sanglezoom * x;
    sy = sanglezoom * y;
    dstwidthhalf = max((int)
		       ceil(max(max(max(fabs(cx + sy), fabs(cx - sy)), fabs(-cx + sy)), fabs(-cx - sy))), 1);
    dstheighthalf = max((int)
			ceil(max(max(max(fabs(sx + cy), fabs(sx - cy)), fabs(-sx + cy)), fabs(-sx - cy))), 1);
    *dstwidth = 2 * dstwidthhalf;
    *dstheight = 2 * dstheighthalf;
} /* rotate_surface_size */ 




SDL_Surface *frame_text_draw(SDL_Rect *r,
 						     FONTCONTAINER *font,
						     char *text_text,int text_spacing,int text_max_w,int text_frame,
						     int text_h_align,int text_v_align,int text_frame_radius,int text_frame_width,
						     int text_frame_point_x,int text_frame_point_y,
						     int text_r,int text_g,int text_b,int text_a,
						     int text_frame_r1,int text_frame_g1,int text_frame_b1,int text_frame_a1,
						     int text_frame_r2,int text_frame_g2,int text_frame_b2,int text_frame_a2)
{
	SDL_Surface *s;

	if (font!=0) {
		switch(text_frame) {
		case 0:
			/* No frame: */ 
			{
				SDL_Surface *s2;
				SDL_Color c;
				Uint32 back_c;
				
				c.r=text_r;
				c.g=text_g;
				c.b=text_b;
				if (text_max_w>0) s2=font->multiline_text_surface_limited(text_text,0,c,text_spacing,text_max_w);
							 else s2=font->multiline_text_surface(text_text,0,c,text_spacing);

				if (s2!=0) {
					s=SDL_CreateRGBSurface(surface_flags,s2->w,s2->h,32,0,0,0,AMASK);
				} else {
					s=SDL_CreateRGBSurface(surface_flags,0,0,32,0,0,0,AMASK);
				} /* if */ 
				back_c=SDL_MapRGBA(s->format,0,0,0,SDL_ALPHA_TRANSPARENT);
				SDL_FillRect(s,0,back_c);
				SDL_SetAlpha(s,SDL_SRCALPHA,0);

				r->x=0;
				r->y=0;
				if (s2!=0) {
					SDL_Rect r2;
					r2.x=0;
					r2.y=0;
					surface_fader(s2,1.0,1.0,1.0,(text_a/255.0F),0);
					surface_blit_alpha(s2,0,s,&r2);
					SDL_FreeSurface(s2);
					s2=0;
				} /* if */ 

				switch(text_h_align) {
				case 0:	r->x=0;
						break;
				case 2:	r->x=int(-s->w);
						break;
				default:r->x=int(-s->w/2);
				} /* switch */ 
				switch(text_v_align) {
				case 0:	r->y=0;
					break;
				case 2:	r->y=int(-s->h);
					break;
				default:r->y=int(-s->h/2);
				} /* switch */ 
			}
			break;

		case 1:
			/* Square frame: */ 
			{
				SDL_Surface *s2;
				SDL_Color c;
				List<int> x_l,y_l;
				int width;
				Uint32 back_c;
				int radius=text_frame_radius/100;
				if (radius<0) radius=0;

				/* render text: */ 
				c.r=text_r;
				c.g=text_g;
				c.b=text_b;

				if (text_max_w>0) s2=font->multiline_text_surface_limited(text_text,0,c,text_spacing,text_max_w);
							 else s2=font->multiline_text_surface(text_text,0,c,text_spacing);

				/* Create frame: */ 
				width=text_frame_width;
				if (width<1) width=1;
				if (s2!=0) {
					s=SDL_CreateRGBSurface(surface_flags,s2->w+(max(8,radius)+width)*2,
														 s2->h+(max(8,radius)+width)*2,
														 32,0,0,0,AMASK);
				} else {
					s=SDL_CreateRGBSurface(surface_flags,(max(8,radius)+width)*2,
														 (max(8,radius)+width)*2,
														 32,0,0,0,AMASK);
				} /* if */ 
				back_c=SDL_MapRGBA(s->format,0,0,0,SDL_ALPHA_TRANSPARENT);
				SDL_FillRect(s,0,back_c);
				SDL_SetAlpha(s,SDL_SRCALPHA,0);

				/* Compute frame: */ 
				{
					int corner_points,i;
					float angle,inc_angle;
					int add_w=width/2;
					int sub_w=add_w-(width-1);

					corner_points=radius/2;
					inc_angle=float(M_PI/(2*corner_points));

					/* Top: */ 
					x_l.Add(new int(radius+add_w));
					y_l.Add(new int(add_w));

					x_l.Add(new int(s->w-1-radius+sub_w));
					y_l.Add(new int(add_w));

					/* Top-Right corner: */ 
					angle=float(M_PI/2-inc_angle);
					for(i=0;i<corner_points;i++) {
						x_l.Add(new int(s->w-1-radius+sub_w+int(ceil(cos(angle)*radius))));
						y_l.Add(new int(radius+add_w-int(ceil(sin(angle)*radius))));
						angle-=inc_angle;
					} /* for */ 
					
					/* Right: */ 
					x_l.Add(new int(s->w-1+sub_w));
					y_l.Add(new int(s->h-1-radius+sub_w));

					/* Bottom-Right corner: */ 
					angle=-inc_angle;
					for(i=0;i<corner_points;i++) {
						x_l.Add(new int(s->w-1-radius+sub_w+int(ceil(cos(angle)*radius))));
						y_l.Add(new int(s->h-1-radius+sub_w-int(floor(sin(angle)*radius))));
						angle-=inc_angle;
					} /* for */ 

					/* Bottom: */ 
					x_l.Add(new int(radius+add_w));
					y_l.Add(new int(s->h-1+sub_w));

					/* Bottom-Left corner: */ 
					angle=float(-M_PI/2-inc_angle);
					for(i=0;i<corner_points;i++) {
						x_l.Add(new int(radius+add_w+int(floor(cos(angle)*radius))));
						y_l.Add(new int(s->h-1-radius+sub_w-int(floor(sin(angle)*radius))));
						angle-=inc_angle;
					} /* for */ 

					/* Left: */ 
					x_l.Add(new int(add_w));
					y_l.Add(new int(radius+add_w));

					/* Top-Left corner: */ 
					angle=float(M_PI-inc_angle);
					for(i=0;i<corner_points;i++) {
						x_l.Add(new int(radius+add_w+int(floor(cos(angle)*radius))));
						y_l.Add(new int(radius+add_w-int(ceil(sin(angle)*radius))));
						angle-=inc_angle;
					} /* for */ 

					x_l.Add(new int(radius+add_w));
					y_l.Add(new int(add_w));
				}

				/* Draw frame: */ 
				{
					Uint32 c;
					int old_x,old_y,*x_p,*y_p;

					if (text_frame_width>0) {
						c=SDL_MapRGBA(s->format,text_frame_r2,text_frame_g2,text_frame_b2,text_frame_a2);
	
						x_p=x_l.ExtractIni();
						y_p=y_l.ExtractIni();
						old_x=*x_p;
						old_y=*y_p;
						delete x_p;
						delete y_p;
						while(!x_l.EmptyP()) {
							x_p=x_l.ExtractIni();
							y_p=y_l.ExtractIni();
							draw_thick_line(s,old_x,old_y,*x_p,*y_p,c,text_frame_width);
							old_x=*x_p;
							old_y=*y_p;
							delete x_p;
							delete y_p;
						} /* while */ 

					} else {
						c=SDL_MapRGBA(s->format,text_frame_r1,text_frame_g1,text_frame_b1,text_frame_a1);
	
						x_p=x_l.ExtractIni();
						y_p=y_l.ExtractIni();
						old_x=*x_p;
						old_y=*y_p;
						delete x_p;
						delete y_p;
						while(!x_l.EmptyP()) {
							x_p=x_l.ExtractIni();
							y_p=y_l.ExtractIni();
							draw_line(s,old_x,old_y,*x_p,*y_p,c);
							old_x=*x_p;
							old_y=*y_p;
							delete x_p;
							delete y_p;
						} /* while */ 
					} /* if */ 
				}

				/* Fill frame: */ 
				{
					Uint32 c;
				
					c=SDL_MapRGBA(s->format,text_frame_r1,text_frame_g1,text_frame_b1,text_frame_a1);
					fast_area_fill(s,s->w/2,s->h/2,c);
				}

				/* Blit text over the frame: */ 
				if (s2!=0) {
					SDL_Rect r2;
					r2.x=max(8,radius)+width;
					r2.y=max(8,radius)+width;
					surface_fader(s2,1.0,1.0,1.0,(text_a/255.0F),0);
					surface_blit_alpha(s2,0,s,&r2);
					SDL_FreeSurface(s2);
					s2=0;
				} /* if */ 

				switch(text_h_align) {
				case 0:	r->x=0;
						break;
				case 2:	r->x=int(-s->w);
						break;
				default:r->x=int(-s->w/2);
				} /* switch */ 
				switch(text_v_align) {
				case 0:	r->y=0;
					break;
				case 2:	r->y=int(-s->h);
					break;
				default:r->y=int(-s->h/2);
				} /* switch */ 
			}
			break;

		case 2:
			/* Balloon frame: */ 
			{
				SDL_Surface *s2;
				SDL_Color c;
				List<int> x_l,y_l;
				int width;
				Uint32 back_c;
				int radius=text_frame_radius/100;
				if (radius<0) radius=0;

				int hot_x,hot_y;
				int frame_x,frame_y;
				int frame_w,frame_h;
				int point_x,point_y;
				
				/* render text: */ 
				c.r=text_r;
				c.g=text_g;
				c.b=text_b;
				if (text_max_w>0) s2=font->multiline_text_surface_limited(text_text,0,c,text_spacing,text_max_w);
							 else s2=font->multiline_text_surface(text_text,0,c,text_spacing);

				/* Create frame: */ 
				width=text_frame_width;
				if (width<1) width=1;
				{
					int dx;
					int dy;

					if (s2!=0) {
						dx=frame_w=s2->w+(max(8,radius)+width)*2;
						dy=frame_h=s2->h+(max(8,radius)+width)*2;
					} else {
						dx=frame_w=(max(8,radius)+width)*2;
						dy=frame_h=(max(8,radius)+width)*2;
					} /* if */ 

					switch(text_h_align) {
					case 0:	hot_x=0;
						break;
					case 2:	hot_x=dx;
						break;
					default:hot_x=dx/2;
					} /* switch */ 

					switch(text_v_align) {
					case 0:	hot_y=0;
						break;
					case 2:	hot_y=dy;
						break;
					default:hot_y=dy/2;
					} /* switch */ 

					point_x=hot_x+text_frame_point_x/100;
					point_y=hot_y+text_frame_point_y/100;
					frame_x=0;
					frame_y=0;

					if (point_x<0) {
						dx-=point_x;
						hot_x-=point_x;
						frame_x=-point_x;
						point_x=0;
					} /* if */ 
					if (point_x>dx) {
						dx=point_x+1;
					} /* if */ 

					if (point_y<0) {
						dy-=point_y;
						hot_y-=point_y;
						frame_y=-point_y;
						point_y=0;
					} /* if */ 
					if (point_y>dy) {
						dy=point_y+1;
					} /* if */ 

					s=SDL_CreateRGBSurface(surface_flags,dx,dy,32,0,0,0,AMASK);
					back_c=SDL_MapRGBA(s->format,0,0,0,SDL_ALPHA_TRANSPARENT);
					SDL_FillRect(s,0,back_c);
					SDL_SetAlpha(s,SDL_SRCALPHA,0);
				}

				/* Compute frame: */ 
				{
					int corner_points,i;
					float angle,inc_angle;
					int add_w=width/2;
					int sub_w=add_w-(width-1);

					corner_points=radius/2;
					inc_angle=float(M_PI/(2*corner_points));

					/* Top: */ 
					x_l.Add(new int(radius+add_w+frame_x));
					y_l.Add(new int(add_w+frame_y));

					x_l.Add(new int(frame_w-1-radius+sub_w+frame_x));
					y_l.Add(new int(add_w+frame_y));

					/* Top-Right corner: */ 
					angle=float(M_PI/2-inc_angle);
					for(i=0;i<corner_points;i++) {
						x_l.Add(new int(frame_w-1-radius+sub_w+int(ceil(cos(angle)*radius))+frame_x));
						y_l.Add(new int(radius+add_w-int(ceil(sin(angle)*radius))+frame_y));
						angle-=inc_angle;
					} /* for */ 
					
					/* Right: */ 
					x_l.Add(new int(frame_w-1+sub_w+frame_x));
					y_l.Add(new int(frame_h-1-radius+sub_w+frame_y));

					/* Bottom-Right corner: */ 
					angle=-inc_angle;
					for(i=0;i<corner_points;i++) {
						x_l.Add(new int(frame_w-1-radius+sub_w+int(ceil(cos(angle)*radius))+frame_x));
						y_l.Add(new int(frame_h-1-radius+sub_w-int(floor(sin(angle)*radius))+frame_y));
						angle-=inc_angle;
					} /* for */ 

					/* Bottom: */ 
					x_l.Add(new int(radius+add_w+frame_x));
					y_l.Add(new int(frame_h-1+sub_w+frame_y));

					/* Bottom-Left corner: */ 
					angle=float(-M_PI/2-inc_angle);
					for(i=0;i<corner_points;i++) {
						x_l.Add(new int(radius+add_w+int(floor(cos(angle)*radius))+frame_x));
						y_l.Add(new int(frame_h-1-radius+sub_w-int(floor(sin(angle)*radius))+frame_y));
						angle-=inc_angle;
					} /* for */ 

					/* Left: */ 
					x_l.Add(new int(add_w+frame_x));
					y_l.Add(new int(radius+add_w+frame_y));

					/* Top-Left corner: */ 
					angle=float(M_PI-inc_angle);
					for(i=0;i<corner_points;i++) {
						x_l.Add(new int(radius+add_w+int(floor(cos(angle)*radius))+frame_x));
						y_l.Add(new int(radius+add_w-int(ceil(sin(angle)*radius))+frame_y));
						angle-=inc_angle;
					} /* for */ 
				}

				/* Add the point: */ 
				{
					int ox,oy;
					int orig_x1,orig_y1,orig_x2,orig_y2;
					int vx,vy;

					if (s2!=0) {
						ox=frame_x+(s2->w+(max(8,radius)+width)*2)/2;
						oy=frame_y+(s2->h+(max(8,radius)+width)*2)/2;
					} else {
						ox=frame_x+((max(8,radius)+width)*2)/2;
						oy=frame_y+((max(8,radius)+width)*2)/2;
					} /* if */ 

					vx=point_x-ox;
					vy=point_y-oy;

					if (vx!=0 || vy!=0) {
						float angle;

						angle=float(atan2(vy,vx));

						orig_x1=int(ox+8*cos(angle-M_PI/2));
						orig_y1=int(oy+8*sin(angle-M_PI/2));

						orig_x2=int(ox+8*cos(angle+M_PI/2));
						orig_y2=int(oy+8*sin(angle+M_PI/2));

						/* Look for segment collision: */ 
						{
							int i;
							int c0=-1,c1=-1;
							int c0x=0,c0y=0,c1x=0,c1y=0;
							List<int> lx,ly;
							int first_x,first_y;
							int old_x,old_y,*x_p,*y_p;

							lx.Instance(x_l);
							ly.Instance(y_l);
							lx.Rewind();
							ly.Rewind();
							lx.Iterate(x_p);
							ly.Iterate(y_p);
							first_x=old_x=*x_p;
							first_y=old_y=*y_p;
							i=0;
							while(lx.Iterate(x_p) && ly.Iterate(y_p)) {
								if (c0==-1 &&
									segment_collision(old_x,old_y,*x_p,*y_p,
													  orig_x1,orig_y1,point_x,point_y,
													  &c0x,&c0y)) {
									c0=i;
								} /* if */ 
								if (c1==-1 &&
									segment_collision(old_x,old_y,*x_p,*y_p,
													  orig_x2,orig_y2,point_x,point_y,
													  &c1x,&c1y)) {
									c1=i;
								} /* if */ 

								old_x=*x_p;
								old_y=*y_p;
								i++;
							} /* while */ 
							if (c0==-1 &&
								segment_collision(old_x,old_y,first_x,first_y,
												  orig_x1,orig_y1,point_x,point_y,
												  &c0x,&c0y)) {
								c0=i;
							} /* if */ 
							if (c1==-1 &&
								segment_collision(old_x,old_y,first_x,first_y,
												  orig_x2,orig_y2,point_x,point_y,
												  &c1x,&c1y)) {
								c1=i;
							} /* if */ 

							if (c0!=-1 && c1!=-1) {
								int i;
								List<int> nlx,nly;
								/* Create a new-modified point list: */ 
								if (c0<=c1) {
									for(i=-1;i<c0;i++) {
										nlx.Add(x_l.ExtractIni());
										nly.Add(y_l.ExtractIni());
									} /* for */ 
									nlx.Add(new int(c0x));
									nly.Add(new int(c0y));
									nlx.Add(new int(point_x));
									nly.Add(new int(point_y));
									nlx.Add(new int(c1x));
									nly.Add(new int(c1y));
									for(;i<c1;i++) {
										delete x_l.ExtractIni();
										delete y_l.ExtractIni();
									} /* for */ 
									while(!x_l.EmptyP()) {
										nlx.Add(x_l.ExtractIni());
										nly.Add(y_l.ExtractIni());
									} /* while */ 
								} else {
									nlx.Add(new int(c0x));
									nly.Add(new int(c0y));
									nlx.Add(new int(point_x));
									nly.Add(new int(point_y));
									nlx.Add(new int(c1x));
									nly.Add(new int(c1y));
									for(i=-1;i<c1;i++) {
										delete x_l.ExtractIni();
										delete y_l.ExtractIni();
									} /* for */ 
									for(;i<c0;i++) {
										nlx.Add(x_l.ExtractIni());
										nly.Add(y_l.ExtractIni());
									} /* for */ 
									while(!x_l.EmptyP()) {
										delete x_l.ExtractIni();
										delete y_l.ExtractIni();
									} /* for */ 
								} /* if */ 
								x_l.Instance(nlx);
								y_l.Instance(nly);
								nlx.SetNoOriginal();
								nly.SetNoOriginal();
								x_l.SetOriginal();
								y_l.SetOriginal();
							} /* if */ 
						}
					} /* if */ 
				}

				/* Draw frame: */ 
				{
					Uint32 c;
					int first_x,first_y;
					int old_x,old_y,*x_p,*y_p;

					x_p=x_l.ExtractIni();
					y_p=y_l.ExtractIni();
					first_x=old_x=*x_p;
					first_y=old_y=*y_p;
					delete x_p;
					delete y_p;

					if (text_frame_width>0) {
						c=SDL_MapRGBA(s->format,text_frame_r2,text_frame_g2,text_frame_b2,text_frame_a2);

						while(!x_l.EmptyP()) {
							x_p=x_l.ExtractIni();
							y_p=y_l.ExtractIni();
							draw_thick_line(s,old_x,old_y,*x_p,*y_p,c,text_frame_width);
							old_x=*x_p;
							old_y=*y_p;
							delete x_p;
							delete y_p;
						} /* while */ 
						draw_thick_line(s,old_x,old_y,first_x,first_y,c,text_frame_width);

					} else {
						c=SDL_MapRGBA(s->format,text_frame_r1,text_frame_g1,text_frame_b1,text_frame_a1);
	
						while(!x_l.EmptyP()) {
							x_p=x_l.ExtractIni();
							y_p=y_l.ExtractIni();
							draw_line(s,old_x,old_y,*x_p,*y_p,c);
							old_x=*x_p;
							old_y=*y_p;
							delete x_p;
							delete y_p;
						} /* while */ 
						draw_line(s,old_x,old_y,first_x,first_y,c);
					} /* if */ 
				}

				/* Fill frame: */ 
				{
					Uint32 c;
				
					c=SDL_MapRGBA(s->format,text_frame_r1,text_frame_g1,text_frame_b1,text_frame_a1);
					fast_area_fill(s,frame_x+width+radius,frame_y+width+radius,c);
				}

				/* Blit text over the frame: */ 
				if (s2!=0) {
					SDL_Rect r2;
					r2.x=max(8,radius)+width+frame_x;
					r2.y=max(8,radius)+width+frame_y;
					surface_fader(s2,1.0,1.0,1.0,(text_a/255.0F),0);
					surface_blit_alpha(s2,0,s,&r2);
					SDL_FreeSurface(s2);
					s2=0;
				} /* if */ 

				r->x=int(-hot_x);
				r->y=int(-hot_y);
			}
			break;
		} /* switch */ 

		return s;
	} /* if */ 

	return 0;
} /* frame_text_draw */ 



void frame_text_bbox(SDL_Rect *r,
					 class FONTCONTAINER *font,
					 char *text_text,int text_spacing,int text_max_w,int text_frame,
					 int text_h_align,int text_v_align,int text_frame_radius,int text_frame_width,
					 int text_frame_point_x,int text_frame_point_y)
{
	SDL_Color c;

	c.r=0;
	c.g=0;
	c.b=0;

	if (font!=0) {
		switch(text_frame) {
		case 0:
			/* No frame: */ 
			{
				SDL_Surface *s2;
				
				if (text_max_w>0) s2=font->multiline_text_surface_limited(text_text,0,c,text_spacing,text_max_w);
							 else s2=font->multiline_text_surface(text_text,0,c,text_spacing);

				if (s2!=0) {
					switch(text_h_align) {
					case 0:	r->x=0;
							break;
					case 2:	r->x=int(-s2->w);
							break;
					default:r->x=int(-s2->w/2);
					} /* switch */ 
					switch(text_v_align) {
					case 0:	r->y=0;
						break;
					case 2:	r->y=int(-s2->h);
						break;
					default:r->y=int(-s2->h/2);
					} /* switch */ 
					r->w=s2->w;
					r->h=s2->h;

					SDL_FreeSurface(s2);
					s2=0;
				} else {
					switch(text_h_align) {
					case 0:	r->x=0;
							break;
					case 2:	r->x=0;
							break;
					default:r->x=0;
					} /* switch */ 
					switch(text_v_align) {
					case 0:	r->y=0;
						break;
					case 2:	r->y=0;
						break;
					default:r->y=0;
					} /* switch */ 
					r->w=0;
					r->h=0;
				} /* if */ 
			}
			break;

		case 1:
			/* Square frame: */ 
			{
				SDL_Surface *s2;
				int width;
				int radius=text_frame_radius/100;
				if (radius<0) radius=0;

				/* Create frame: */ 
				width=text_frame_width;
				if (width<1) width=1;

				if (text_max_w>0) s2=font->multiline_text_surface_limited(text_text,0,c,text_spacing,text_max_w);
							 else s2=font->multiline_text_surface(text_text,0,c,text_spacing);

				 if (s2!=0) {
					r->w=s2->w+(max(8,radius)+width)*2;
					r->h=s2->h+(max(8,radius)+width)*2;
					SDL_FreeSurface(s2);
					s2=0;
				 } else {
					r->w=(max(8,radius)+width)*2;
					r->h=(max(8,radius)+width)*2;
				 } /* if */ 


				switch(text_h_align) {
				case 0:	r->x=0;
						break;
				case 2:	r->x=int(-r->w);
						break;
				default:r->x=int(-r->w/2);
				} /* switch */ 
				switch(text_v_align) {
				case 0:	r->y=0;
					break;
				case 2:	r->y=int(-r->h);
					break;
				default:r->y=int(-r->h/2);
				} /* switch */ 
	
			}
			break;

		case 2:
			/* Balloon frame: */ 
			{
				SDL_Surface *s2;
				int width;
				int radius=text_frame_radius/100;
				if (radius<0) radius=0;

				int hot_x,hot_y;
				int frame_x,frame_y;
				int frame_w,frame_h;
				int point_x,point_y;
				

				/* Create frame: */ 
				width=text_frame_width;
				if (width<1) width=1;

				if (text_max_w>0) s2=font->multiline_text_surface_limited(text_text,0,c,text_spacing,text_max_w);
							 else s2=font->multiline_text_surface(text_text,0,c,text_spacing);

				if (s2!=0) {
					r->w=frame_w=s2->w+(max(8,radius)+width)*2;
					r->h=frame_h=s2->h+(max(8,radius)+width)*2;
					SDL_FreeSurface(s2);
					s2=0;
				} else {
					r->w=frame_w=(max(8,radius)+width)*2;
					r->h=frame_h=(max(8,radius)+width)*2;
				} /* if */ 


				switch(text_h_align) {
				case 0:	hot_x=0;
					break;
				case 2:	hot_x=r->w;
					break;
				default:hot_x=r->w/2;
				} /* switch */ 

				switch(text_v_align) {
				case 0:	hot_y=0;
					break;
				case 2:	hot_y=r->h;
					break;
				default:hot_y=r->h/2;
				} /* switch */ 

				point_x=hot_x+text_frame_point_x/100;
				point_y=hot_y+text_frame_point_y/100;
				frame_x=0;
				frame_y=0;

				if (point_x<0) {
					r->w-=point_x;
					hot_x-=point_x;
					frame_x=-point_x;
					point_x=0;
				} /* if */ 
				if (point_x>r->w) {
					r->w=point_x+1;
				} /* if */ 

				if (point_y<0) {
					r->h-=point_y;
					hot_y-=point_y;
					frame_y=-point_y;
					point_y=0;
				} /* if */ 
				if (point_y>r->h) {
					r->h=point_y+1;
				} /* if */ 
		
				r->x=int(-hot_x);
				r->y=int(-hot_y);
			}
			break;
		} /* switch */ 

	} else {
		r->x=0;
		r->y=0;
		r->w=0;
		r->h=0;
	} /* if */ 

} /* frame_text_bbox */ 



// Key names:

char *joystick_names[8][12]={{"JOY1_LEFT","JOY1_RIGHT","JOY1_UP","JOY1_DOWN",
	 						  "JOY1_BUTTON1","JOY1_BUTTON2","JOY1_BUTTON3","JOY1_BUTTON4",
							  "JOY1_BUTTON5","JOY1_BUTTON6","JOY1_BUTTON7","JOY1_BUTTON8"},
							{"JOY2_LEFT","JOY2_RIGHT","JOY2_UP","JOY1_DOWN", 
							 "JOY2_BUTTON1","JOY2_BUTTON2","JOY2_BUTTON3","JOY2_BUTTON4",
							 "JOY2_BUTTON5","JOY2_BUTTON6","JOY2_BUTTON7","JOY2_BUTTON8"},
							{"JOY3_LEFT","JOY3_RIGHT","JOY3_UP","JOY1_DOWN",
							 "JOY3_BUTTON1","JOY3_BUTTON2","JOY3_BUTTON3","JOY3_BUTTON4",
							 "JOY3_BUTTON5","JOY3_BUTTON6","JOY3_BUTTON7","JOY3_BUTTON8"},
							{"JOY4_LEFT","JOY4_RIGHT","JOY4_UP","JOY1_DOWN",
							 "JOY4_BUTTON1","JOY4_BUTTON2","JOY4_BUTTON3","JOY4_BUTTON4",
							 "JOY4_BUTTON5","JOY4_BUTTON6","JOY4_BUTTON7","JOY4_BUTTON8"},
							{"JOY5_LEFT","JOY5_RIGHT","JOY5_UP","JOY1_DOWN",
							 "JOY5_BUTTON1","JOY5_BUTTON2","JOY5_BUTTON3","JOY5_BUTTON4",
							 "JOY5_BUTTON5","JOY5_BUTTON6","JOY5_BUTTON7","JOY5_BUTTON8"},
							{"JOY6_LEFT","JOY6_RIGHT","JOY6_UP","JOY1_DOWN",
							 "JOY6_BUTTON1","JOY6_BUTTON2","JOY6_BUTTON3","JOY6_BUTTON4",
							 "JOY6_BUTTON5","JOY6_BUTTON6","JOY6_BUTTON7","JOY6_BUTTON8"},
							{"JOY7_LEFT","JOY7_RIGHT","JOY7_UP","JOY1_DOWN",
							 "JOY7_BUTTON1","JOY7_BUTTON2","JOY7_BUTTON3","JOY7_BUTTON4",
							 "JOY7_BUTTON5","JOY7_BUTTON6","JOY7_BUTTON7","JOY7_BUTTON8"},
							{"JOY8_LEFT","JOY8_RIGHT","JOY8_UP","JOY1_DOWN",
							 "JOY8_BUTTON1","JOY8_BUTTON2","JOY8_BUTTON3","JOY8_BUTTON4",
							 "JOY8_BUTTON5","JOY8_BUTTON6","JOY8_BUTTON7","JOY8_BUTTON8"}};

char *other_joystick_names[12]={"JOY#_LEFT","JOY#_RIGHT","JOY#_UP","JOY#_DOWN",
							    "JOY#_BUTTON1","JOY#_BUTTON2","JOY#_BUTTON3","JOY#_BUTTON4",
							    "JOY#_BUTTON5","JOY#_BUTTON6","JOY#_BUTTON7","JOY#_BUTTON8"};

char *defined_keys[9]={"[LEFT]","[RIGHT]","[UP]","[DOWN]","[FIRE1]","[FIRE2]","[FIRE3]","[FIRE4]","[ANY]"};
char *unknown_key="???";				


char *input_key_name(int key)
{
	if (key>=0 && key<SDLK_LAST) {
		return SDL_GetKeyName((SDLKey)key);
	} else {
		if (key<0) {
			if (key>-10) {
				return defined_keys[-1-key];
			} else {
				return unknown_key;
			} /* if */ 
		} else {
			int joy=(key-SDLK_LAST)/12;
			if (joy<8) {
				return joystick_names[joy][(key-SDLK_LAST)%12];
			} else {
				return other_joystick_names[(key-SDLK_LAST)%12];
			} /* if */ 
		} /* if */ 
	} /* if */ 
} /* input_jey_name */ 
