#ifndef __THE_MAZE_OF_GALIOUS_2_TILES
#define __THE_MAZE_OF_GALIOUS_2_TILES


/* MOG2 TILE CLASS: */ 

class TILE {
	// The EDITOR classes can access all the MOG2 members
	friend class RObjectWnd;
	friend class REditWnd;
	friend class RToolsWnd;
	friend class OBitmapWnd;
	friend class OObjectBmpWnd;
	friend class GrabGrabberWnd;
	friend class BAnimEditorWnd;
	friend class BTileViewWnd;
	friend class BTileSelectionWnd;
	friend class HotSpotWnd;
	friend class LinksWnd;
	friend class OPartsWnd;
	friend class IObjectWnd;
	friend class IToolsWnd;
	friend class IEditWnd;

//	friend class ActiveMOG2;
//	friend class ActiveROOM;

//	friend class OBJECT;
//	friend class PLACED_OBJECT;
//	friend class PLACED_TILE;
//	friend class ActiveOBJECT;

public:
	TILE(void);
	TILE(int x,int y,int dx,int dy,SDL_Surface *orig,char *orig_name);
	~TILE(void);

	bool save(FILE *fp);
	bool load(FILE *fp,class MOG2 *game);

	void draw(int x,int y,SDL_Surface *dest);
	void draw_transform(int x,int y,SDL_Surface *dest,float scale,float angle);
	void draw_shaded(int x,int y,SDL_Surface *dest,int factor,int r,int g,int b,int a);
	void draw_shaded_transform(int x,int y,SDL_Surface *dest,float scale,float angle,int factor,int r,int g,int b,int a);
	void draw_bicolor(int x,int y,SDL_Surface *dest,int factor,int r1,int g1,int b1,int a1,int r2,int g2,int b2,int a2);
	void draw_bicolor_transform(int x,int y,SDL_Surface *dest,float scale,float angle,int factor,int r1,int g1,int b1,int a1,int r2,int g2,int b2,int a2);
	void draw_reflection(int x,int y,SDL_Surface *dest,int reflection_kh,int reflection_kv,int reflection_oh,int reflection_ov,int reflection_wave_height,int center_x,int center_y,int factor);
	void draw_reflection_transform(int x,int y,SDL_Surface *dest,float scale,float angle,int reflection_kh,int reflection_kv,int reflection_oh,int reflection_ov,int reflection_wave_height,int center_x,int center_y,int factor);
	void draw_refraction(int x,int y,SDL_Surface *dest,int refraction_distance,int refraction_kh,int refraction_kv,int refraction_oh,int refraction_ov,int refraction_wave_height,int factor);
	void draw_refraction_transform(int x,int y,SDL_Surface *dest,float scale,float angle,int refraction_distance,int refraction_kh,int refraction_kv,int refraction_oh,int refraction_ov,int refraction_wave_height,int factor);

	void draw_collision_mask(int x,int y,SDL_Surface *dest);
	void draw_collision_mask_transform(int x,int y,SDL_Surface *dest,float scale,float angle);

	void draw_mask(int x,int y,SDL_Surface *dest);

	class MOG2CollisionData *get_collision_data(float scale,float angle);

	int get_dx() {return orig->w;};
	int get_dy() {return orig->h;};

	void clear(void);
	void free(void);
	void instance(TILE *t);

private:

	SDL_Rect r;
	SDL_Surface *orig,*mask_visualization,*mask_collision;
	MOG2CollisionData *base_collision_data,*transform_collision_data;
	float tcd_scale,tcd_angle;	/* These variables keep the scale and angle with which  */ 
								/* "transform_collision_data" was computed.				*/ 

	char *orig_name;
	int mask_type,collision_mask_type;
};


/* TILE_SOURCE CLASS: */ 

class TILE_SOURCE {
	// The EDITOR classes can access all the MOG2 members
	friend class GrabGrabberWnd;
	friend class OBitmapWnd;

	friend class TILE;

public:
	TILE_SOURCE(void);
	TILE_SOURCE(char *filename);
	~TILE_SOURCE(void);

	bool save(FILE *fp);
	bool load(FILE *fp);

	bool cmp(char *n);
	bool cmp(TILE_SOURCE *t);

private:
	SDL_Surface *sfc;
	char *fname;
};



/* PLACED TILES CLASS: */ 

class PLACED_TILE {
	// The EDITOR classes can access all the MOG2 members
	friend class EDITOR;
	friend class REditWnd;
	friend class RToolsWnd;
	friend class BRMoveDlg;
	friend class IToolsWnd;
	friend class IEditWnd;

	friend class ROOM;

public:
	PLACED_TILE(void);
	PLACED_TILE(long int x,long int y,TILE *t);
	PLACED_TILE(PLACED_TILE &t);
	~PLACED_TILE(void);

	bool save(FILE *fp,MOG2 *game);
	bool load(FILE *fp,MOG2 *game);

	void draw(SDL_Surface *sfc);
	void draw(long int sx,long int sy,SDL_Surface *sfc);
	void draw_collision_mask(long int sx,long int sy,SDL_Surface *sfc);
	void draw_scaled(long int sx,long int sy,SDL_Surface *dest,float scale);

	void draw(long int sx,long int sy,SDL_Surface *sfc,SDL_Rect vp);
	void draw_collision_mask(long int sx,long int sy,SDL_Surface *sfc,SDL_Rect vp);

	int get_x() {return x;};
	int get_y() {return y;};

private:
	long int x,y;
	TILE *tile;
};

#endif

