#ifdef _WIN32
#include "windows.h"
#include "direct.h"
#endif

#include "string.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "animscript.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BListWnd.h"
#include "BIntegerEdDlg.h"

#include "ParallaxWnd.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

BParallaxWnd::BParallaxWnd(int x,int y,int dx,int dy,ROOM *r,TTF_Font *font) : BActiveWindow(x,y,dx,dy,font)
{
	BButton *b;
	int but_dx,but_dx2;
	int i;

	but_dx=get_text_width("ACCEPT",font)+10;
	b=new BButton(10,4,but_dx,12,"ACCEPT",1);
	buttons.Add(b);

	but_dx2=get_text_width("CANCEL",font)+10;
	b=new BButton(20+but_dx,4,but_dx2,12,"CANCEL",2);
	buttons.Add(b);

	room=r;

	for(i=0;i<MAX_OBJECT_PLANES;i++) {
		plane_h_speed[i]=room->plane_h_speed[i];
		plane_v_speed[i]=room->plane_v_speed[i];
		plane_h_align[i]=room->plane_h_align[i];
		plane_v_align[i]=room->plane_v_align[i];
	} /* for */ 
	drawing_order=room->drawing_order;
	h_circularity=room->horizontal_circularity;
	v_circularity=room->vertical_circularity;

	c_grid_dx=room->COLLISION_GRID_SX;
	c_grid_dy=room->COLLISION_GRID_SY;
	c_grid_tolerance=room->COLLISION_GRID_TOLERANCE;

	lw=new BListWnd(10,30,100,100,&plane_names,font);
} /* BParallaxWnd::BParallaxWnd */ 


BParallaxWnd::~BParallaxWnd()
{
	delete lw;
} /* BParallaxWnd::BParallaxWnd */ 


void BParallaxWnd::contens_size(int *dx,int *dy)
{
	*dx=old_dx;
	*dy=old_dy;
} /* BParallaxWnd::contens_size */ 



bool BParallaxWnd::cycle(int stx,int sty)
{
	int selected;

	if (!BActiveWindow::cycle(stx,sty)) return false;

	while(plane_names.Length()>room->used_object_planes) {
		delete [](plane_names.Extract());
		lw->set_dirty();
	} /* while */ 
	while(plane_names.Length()<room->used_object_planes) {
		char tmp[80],*tmp2;

		sprintf(tmp,"Plane %i",plane_names.Length());
		tmp2=new char[strlen(tmp)+1];
		strcpy(tmp2,tmp);
		plane_names.Add(tmp2);
		
		lw->set_dirty();
	} /* while */ 

	lw->cycle(stx+10,sty+30);
	if (lw->is_dirty()) set_dirty();

	selected=lw->get_selected();
	if (selected!=-1 &&
		((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {

		if (local_mouse_y>=30 && local_mouse_y<=30+TTF_FontHeight(font) &&
			local_mouse_x>=120 && local_mouse_x<=120+get_text_width("H. Speed: 99999",font)) {
			BIntegerEdDlg *dlg;
			int x,y;

			new_window_coordinates(140,60,&x,&y);
			dlg=new BIntegerEdDlg(x,y,140,60,"Enter H. Speed:",&(plane_h_speed[selected]),5,font);
			dlg->Execute();
			delete dlg;
			set_dirty();
		} /* if */ 

		if (local_mouse_y>=40 && local_mouse_y<=40+TTF_FontHeight(font) &&
			local_mouse_x>=120 && local_mouse_x<=120+get_text_width("V. Speed: 99999",font)) {
			BIntegerEdDlg *dlg;
			int x,y;

			new_window_coordinates(140,60,&x,&y);
			dlg=new BIntegerEdDlg(x,y,140,60,"Enter H. Speed:",&(plane_v_speed[selected]),5,font);
			dlg->Execute();
			delete dlg;
			set_dirty();
		} /* if */ 

		if (local_mouse_y>=60 && local_mouse_y<=60+TTF_FontHeight(font) &&
			local_mouse_x>=120 && local_mouse_x<=120+get_text_width("H. Align: CENTER",font)) {

			plane_h_align[selected]++;
			if (plane_h_align[selected]>=3) plane_h_align[selected]=0;

			set_dirty();
		} /* if */ 
		if (local_mouse_y>=70 && local_mouse_y<=70+TTF_FontHeight(font) &&
			local_mouse_x>=120 && local_mouse_x<=120+get_text_width("V. Align: CENTER",font)) {

			plane_v_align[selected]++;
			if (plane_v_align[selected]>=3) plane_v_align[selected]=0;

			set_dirty();
		} /* if */ 

	} /* if */ 

	if (((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		if (local_mouse_y>=90 && local_mouse_y<=90+TTF_FontHeight(font) &&
			local_mouse_x>=120 && local_mouse_x<=120+get_text_width("D. Order: LEFTRIGHT",font)) {

			drawing_order++;
			if (drawing_order>=5) drawing_order=0;

			set_dirty();
		} /* if */ 
	} /* if */ 

	if (((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		if (local_mouse_y>=100 && local_mouse_y<=100+TTF_FontHeight(font) &&
			local_mouse_x>=120 && local_mouse_x<=120+get_text_width("H. Circularity: FALSE",font)) {

			if (h_circularity) h_circularity=false;
						  else h_circularity=true;
			set_dirty();
		} /* if */ 
	} /* if */ 
	if (((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		if (local_mouse_y>=110 && local_mouse_y<=110+TTF_FontHeight(font) &&
			local_mouse_x>=120 && local_mouse_x<=120+get_text_width("V. Circularity: FALSE",font)) {

			if (v_circularity) v_circularity=false;
						  else v_circularity=true;
			set_dirty();
		} /* if */ 
	} /* if */ 

	if (((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		if (local_mouse_y>=130 && local_mouse_y<=130+TTF_FontHeight(font) &&
			local_mouse_x>=120 && local_mouse_x<=120+get_text_width("C.Grid DX: 999",font)) {
			/* Change GRID_DX: */ 
			BIntegerEdDlg *dlg;
			int y,x;

			new_window_coordinates(140,60,&x,&y);

			dlg=new BIntegerEdDlg(x,y,140,60,"Enter grid DX:",&c_grid_dx,4,font);
			dlg->Execute();
			delete dlg;

			set_dirty();
		} /* if */ 
	} /* if */ 

	if (((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		if (local_mouse_y>=140 && local_mouse_y<=140+TTF_FontHeight(font) &&
			local_mouse_x>=120 && local_mouse_x<=120+get_text_width("C.Grid DY: 999",font)) {
			/* Change GRID_DY: */ 
			BIntegerEdDlg *dlg;
			int y,x;

			new_window_coordinates(140,60,&x,&y);

			dlg=new BIntegerEdDlg(x,y,140,60,"Enter grid DY:",&c_grid_dy,4,font);
			dlg->Execute();
			delete dlg;

			set_dirty();
		} /* if */ 
	} /* if */ 

	if (((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		if (local_mouse_y>=150 && local_mouse_y<=150+TTF_FontHeight(font) &&
			local_mouse_x>=120 && local_mouse_x<=120+get_text_width("C.Grid Tolerance: 999",font)) {
			/* Change GRID_TOLERANCE: */ 
			BIntegerEdDlg *dlg;
			int y,x;

			new_window_coordinates(140,60,&x,&y);

			dlg=new BIntegerEdDlg(x,y,140,60,"Enter grid Tolerance:",&c_grid_tolerance,4,font);
			dlg->Execute();
			delete dlg;

			set_dirty();
		} /* if */ 
	} /* if */ 


	return true;
} /* BParallaxWnd::cycle */ 



void BParallaxWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_Rect r;
	int selected;
	char *d_order[5]={"NONE", "TOPDOWN", "BOTTOMUP", "LEFTRIGHT", "RIGHTLEFT"};
	char tmp[80];
	char *h_align[3]={"LEFT", "CENTER", "RIGHT"};
	char *v_align[3]={"TOP", "CENTER", "BOTTOM"};

	SDL_FillRect(sfc,0,0x0000007f);

	r.x=1-stx;
	r.y=1-sty;
	r.w=dx-2;
	r.h=dy-2;
	SDL_FillRect(sfc,&r,0x000000ff);

	BActiveWindow::draw_contens(sfc,stx,sty,dx,dy);

	lw->draw(sfc,10-stx,30-sty,100,100);

	selected=lw->get_selected();
	if (selected!=-1) {
		/* Print parallax information: */ 

		sprintf(tmp,"H. Speed: %i",plane_h_speed[selected]);
		print_left(tmp,font,255,255,255,sfc,120,30);
		sprintf(tmp,"V. Speed: %i",plane_v_speed[selected]);
		print_left(tmp,font,255,255,255,sfc,120,40);
		sprintf(tmp,"H. Align: %s",h_align[plane_h_align[selected]]);
		print_left(tmp,font,255,255,255,sfc,120,60);
		sprintf(tmp,"V. Align: %s",v_align[plane_v_align[selected]]);
		print_left(tmp,font,255,255,255,sfc,120,70);
	} /* if */ 

	sprintf(tmp,"D. Order: %s",d_order[drawing_order]);
	print_left(tmp,font,255,255,255,sfc,120,90);
	if (h_circularity) print_left("H. Circularity: TRUE",font,255,255,255,sfc,120,100);
				  else print_left("H. Circularity: FALSE",font,255,255,255,sfc,120,100);
	if (v_circularity) print_left("V. Circularity: TRUE",font,255,255,255,sfc,120,110);
				  else print_left("V. Circularity: FALSE",font,255,255,255,sfc,120,110);

	sprintf(tmp,"C.Grid DX: %i",c_grid_dx);
	print_left(tmp,font,255,255,255,sfc,120,130);
	sprintf(tmp,"C.Grid DY: %i",c_grid_dy);
	print_left(tmp,font,255,255,255,sfc,120,140);
	sprintf(tmp,"C.Grid Tolerance: %i",c_grid_tolerance);
	print_left(tmp,font,255,255,255,sfc,120,150);

} /* BParallaxWnd::draw_contens */ 


bool BParallaxWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		{
			int i;
			for(i=0;i<MAX_OBJECT_PLANES;i++) {
				room->plane_h_speed[i]=plane_h_speed[i];
				room->plane_v_speed[i]=plane_v_speed[i];
				room->plane_h_align[i]=plane_h_align[i];
				room->plane_v_align[i]=plane_v_align[i];
			} /* for */ 
			room->drawing_order=drawing_order;
			room->horizontal_circularity=h_circularity;
			room->vertical_circularity=v_circularity;
			room->COLLISION_GRID_SX=c_grid_dx;
			room->COLLISION_GRID_SY=c_grid_dy;
			room->COLLISION_GRID_TOLERANCE=c_grid_tolerance;
			return false;
		}
		break;

	case 2:
		return false;
		break;

	} /* switch */ 

	return false;
} /* BParallaxWnd::button_pressed */ 

